/*
 * display.c
 *
 *  Created on: 29.08.2016
 *      Author: wn
 */


#include <msp430g2553.h>
#include <stdint.h>
#include <intrinsics.h>

#include "display.h"
#include "gpio.h"
#include "time.h"


const uint32_t DISPLAY_CYCLE = 5;

const uint16_t INIT_CYCLE_DELAY = 200;

typedef enum {
  ALL_ID = 10,
  H_ID,
  I_ID,
  EMPTY_ID,
} tPatternId;

const tPin ALL_PATTERN[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin H_PATTERN[] = { SEG_B, SEG_C, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin I_PATTERN[] = { SEG_B, SEG_C, PINS_END };
const tPin L_PATTERN[] = { SEG_D, SEG_E, SEG_F, PINS_END };
const tPin O_PATTERN[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, PINS_END };
const tPin EMPTY_PATTERN[] = { PINS_END };
const tPin NUMBER0[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, PINS_END };
const tPin NUMBER1[] = { SEG_B, SEG_C, PINS_END };
const tPin NUMBER2[] = { SEG_A, SEG_B, SEG_G, SEG_E, SEG_D, PINS_END };
const tPin NUMBER3[] = { SEG_A, SEG_B, SEG_G, SEG_C, SEG_D, PINS_END };
const tPin NUMBER4[] = { SEG_F, SEG_G, SEG_B, SEG_C, PINS_END };
const tPin NUMBER5[] = { SEG_A, SEG_F, SEG_G, SEG_C, SEG_D, PINS_END };
const tPin NUMBER6[] = { SEG_A, SEG_C, SEG_D, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin NUMBER7[] = { SEG_A, SEG_B, SEG_C, PINS_END };
const tPin NUMBER8[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin NUMBER9[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_F, SEG_G, PINS_END };

const tPin *NUMBERS[] = { NUMBER0, NUMBER1, NUMBER2, NUMBER3, NUMBER4,
    NUMBER5, NUMBER6, NUMBER7, NUMBER8, NUMBER9, ALL_PATTERN, H_PATTERN,
    I_PATTERN, EMPTY_PATTERN };

const tPin DIGITS[] = { DIGIT_0, DIGIT_1, PINS_END };


uint8_t digitValues[2] = { EMPTY_ID, EMPTY_ID };



void displayInit(void *handleArg) {
  for (tPin d = DIGIT_0; d <= DIGIT_1; d++) {
    gpioSetPin(d, LOW);
    for (tPin s = SEG_A; s <= SEG_G; s++) {
      gpioSetPin(s, HIGH);
      ms_active_delay(INIT_CYCLE_DELAY);
      gpioSetPin(s, LOW);
    }
    gpioSetPin(d, HIGH);
  }
}

static void showNumber(uint8_t n) {
  const tPin *pattern = NUMBERS[ALL_ID];
  do {
    gpioSetPin(*pattern, LOW);
    pattern++;
  } while (*pattern != PINS_END);
  pattern = NUMBERS[n];
  do {
    gpioSetPin(*pattern, HIGH);
    pattern++;
  } while (*pattern != PINS_END);
}

void displaySetValue(uint8_t v) {
  if (v >= 100) {
    digitValues[1] = H_ID;
    digitValues[0] = I_ID;
  } else {
    digitValues[1] = v / 10;
    digitValues[0] = v - digitValues[1] * 10;
  }
}

void displayExec(void *handleArg) {
  static uint8_t activeDigit = 0;

  activeDigit++;
  if (activeDigit == 2) {
    activeDigit = 0;
  }

  const tPin *digit = DIGITS;
  do {
    gpioSetPin(*digit, HIGH);
    digit++;
  } while (*digit != PINS_END);

  digit = DIGITS + activeDigit;
  gpioSetPin(*digit, LOW);

  showNumber(digitValues[activeDigit]);
}
